@@ -7,6 +7,15 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 
+//config:config RPM
+//config:	bool "rpm"
+//config:	default y
+//config:	help
+//config:	  Mini RPM applet - queries and extracts RPM packages.
+
+//applet:IF_RPM(APPLET(rpm, BB_DIR_BIN, BB_SUID_DROP))
+//kbuild:lib-$(CONFIG_RPM) += rpm.o
+
 //usage:#define rpm_trivial_usage
 //usage:       "-i PACKAGE.rpm; rpm -qp[ildc] PACKAGE.rpm"
 //usage:#define rpm_full_usage "\n\n"
@@ -14,13 +23,13 @@
 //usage:     "\nCommands:"
 //usage:     "\n	-i	Install package"
 //usage:     "\n	-qp	Query package"
-//usage:     "\n	-i	Show information"
-//usage:     "\n	-l	List contents"
-//usage:     "\n	-d	List documents"
-//usage:     "\n	-c	List config files"
+//usage:     "\n	-qpi	Show information"
+//usage:     "\n	-qpl	List contents"
+//usage:     "\n	-qpd	List documents"
+//usage:     "\n	-qpc	List config files"
 
 #include "libbb.h"
-#include "archive.h"
+#include "bb_archive.h"
 #include "rpm.h"
 
 #define RPM_CHAR_TYPE           1
@@ -79,136 +88,13 @@ typedef struct {
 	uint32_t count; /* 4 byte count */
 } rpm_index;
 
-static void *map;
-static rpm_index **mytags;
-static int tagcount;
-
-static void extract_cpio(int fd, const char *source_rpm);
-static rpm_index **rpm_gettags(int fd, int *num_tags);
-static int bsearch_rpmtag(const void *key, const void *item);
-static char *rpm_getstr(int tag, int itemindex);
-static int rpm_getint(int tag, int itemindex);
-static int rpm_getcount(int tag);
-static void fileaction_dobackup(char *filename, int fileref);
-static void fileaction_setowngrp(char *filename, int fileref);
-static void loop_through_files(int filetag, void (*fileaction)(char *filename, int fileref));
-
-int rpm_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int rpm_main(int argc, char **argv)
-{
-	int opt = 0, func = 0, rpm_fd, offset;
-	const int pagesize = getpagesize();
-
-	while ((opt = getopt(argc, argv, "iqpldc")) != -1) {
-		switch (opt) {
-		case 'i': /* First arg: Install mode, with q: Information */
-			if (!func) func = rpm_install;
-			else func |= rpm_query_info;
-			break;
-		case 'q': /* First arg: Query mode */
-			if (func) bb_show_usage();
-			func = rpm_query;
-			break;
-		case 'p': /* Query a package */
-			func |= rpm_query_package;
-			break;
-		case 'l': /* List files in a package */
-			func |= rpm_query_list;
-			break;
-		case 'd': /* List doc files in a package (implies list) */
-			func |= rpm_query_list;
-			func |= rpm_query_list_doc;
-			break;
-		case 'c': /* List config files in a package (implies list) */
-			func |= rpm_query_list;
-			func |= rpm_query_list_config;
-			break;
-		default:
-			bb_show_usage();
-		}
-	}
-	argv += optind;
-	//argc -= optind;
-	if (!argv[0]) {
-		bb_show_usage();
-	}
-
-	while (*argv) {
-		const char *source_rpm;
-
-		rpm_fd = xopen(*argv++, O_RDONLY);
-		mytags = rpm_gettags(rpm_fd, &tagcount);
-		if (!mytags)
-			bb_error_msg_and_die("error reading rpm header");
-		offset = xlseek(rpm_fd, 0, SEEK_CUR);
-		/* Mimimum is one page */
-		map = mmap(0, offset > pagesize ? (offset + offset % pagesize) : pagesize, PROT_READ, MAP_PRIVATE, rpm_fd, 0);
-
-		source_rpm = rpm_getstr(TAG_SOURCERPM, 0);
-
-		if (func & rpm_install) {
-			/* Backup any config files */
-			loop_through_files(TAG_BASENAMES, fileaction_dobackup);
-			/* Extact the archive */
-			extract_cpio(rpm_fd, source_rpm);
-			/* Set the correct file uid/gid's */
-			loop_through_files(TAG_BASENAMES, fileaction_setowngrp);
-		}
-		else if ((func & (rpm_query|rpm_query_package)) == (rpm_query|rpm_query_package)) {
-			if (!(func & (rpm_query_info|rpm_query_list))) {
-				/* If just a straight query, just give package name */
-				printf("%s-%s-%s\n", rpm_getstr(TAG_NAME, 0), rpm_getstr(TAG_VERSION, 0), rpm_getstr(TAG_RELEASE, 0));
-			}
-			if (func & rpm_query_info) {
-				/* Do the nice printout */
-				time_t bdate_time;
-				struct tm *bdate_ptm;
-				char bdatestring[50];
-				const char *p;
-
-				p = rpm_getstr(TAG_PREFIXS, 0);
-				if (!p) p = "(not relocateable)";
-				printf("Name        : %-29sRelocations: %s\n", rpm_getstr(TAG_NAME, 0), p);
-				p = rpm_getstr(TAG_VENDOR, 0);
-				if (!p) p = "(none)";
-				printf("Version     : %-34sVendor: %s\n", rpm_getstr(TAG_VERSION, 0), p);
-				bdate_time = rpm_getint(TAG_BUILDTIME, 0);
-				bdate_ptm = localtime(&bdate_time);
-				strftime(bdatestring, 50, "%a %d %b %Y %T %Z", bdate_ptm);
-				printf("Release     : %-30sBuild Date: %s\n", rpm_getstr(TAG_RELEASE, 0), bdatestring);
-				printf("Install date: %-30sBuild Host: %s\n", "(not installed)", rpm_getstr(TAG_BUILDHOST, 0));
-				printf("Group       : %-30sSource RPM: %s\n", rpm_getstr(TAG_GROUP, 0), source_rpm);
-				printf("Size        : %-33dLicense: %s\n", rpm_getint(TAG_SIZE, 0), rpm_getstr(TAG_LICENSE, 0));
-				printf("URL         : %s\n", rpm_getstr(TAG_URL, 0));
-				printf("Summary     : %s\n", rpm_getstr(TAG_SUMMARY, 0));
-				printf("Description :\n%s\n", rpm_getstr(TAG_DESCRIPTION, 0));
-			}
-			if (func & rpm_query_list) {
-				int count, it, flags;
-				count = rpm_getcount(TAG_BASENAMES);
-				for (it = 0; it < count; it++) {
-					flags = rpm_getint(TAG_FILEFLAGS, it);
-					switch (func & (rpm_query_list_doc|rpm_query_list_config)) {
-					case rpm_query_list_doc:
-						if (!(flags & RPMFILE_DOC)) continue;
-						break;
-					case rpm_query_list_config:
-						if (!(flags & RPMFILE_CONFIG)) continue;
-						break;
-					case rpm_query_list_doc|rpm_query_list_config:
-						if (!(flags & (RPMFILE_CONFIG|RPMFILE_DOC))) continue;
-						break;
-					}
-					printf("%s%s\n",
-						rpm_getstr(TAG_DIRNAMES, rpm_getint(TAG_DIRINDEXES, it)),
-						rpm_getstr(TAG_BASENAMES, it));
-				}
-			}
-		}
-		free(mytags);
-	}
-	return 0;
-}
+struct globals {
+	void *map;
+	rpm_index **mytags;
+	int tagcount;
+} FIX_ALIASING;
+#define G (*(struct globals*)&bb_common_bufsiz1)
+#define INIT_G() do { } while (0)
 
 static void extract_cpio(int fd, const char *source_rpm)
 {
@@ -231,12 +117,12 @@ static void extract_cpio(int fd, const char *source_rpm)
 		/* compat: overwrite existing files.
 		 * try "rpm -i foo.src.rpm" few times in a row -
 		 * standard rpm will not complain.
-		 * (TODO? real rpm creates "file;1234" and then renames it) */
-		| ARCHIVE_UNLINK_OLD;
+		 */
+		| ARCHIVE_REPLACE_VIA_RENAME;
 	archive_handle->src_fd = fd;
 	/*archive_handle->offset = 0; - init_handle() did it */
 
-	setup_unzip_on_fd(archive_handle->src_fd /*, fail_if_not_detected: 1*/);
+	setup_unzip_on_fd(archive_handle->src_fd, /*fail_if_not_compressed:*/ 1);
 	while (get_header_cpio(archive_handle) == EXIT_SUCCESS)
 		continue;
 }
@@ -294,7 +180,7 @@ static int bsearch_rpmtag(const void *key, const void *item)
 static int rpm_getcount(int tag)
 {
 	rpm_index **found;
-	found = bsearch(&tag, mytags, tagcount, sizeof(struct rpmtag *), bsearch_rpmtag);
+	found = bsearch(&tag, G.mytags, G.tagcount, sizeof(struct rpmtag *), bsearch_rpmtag);
 	if (!found)
 		return 0;
 	return found[0]->count;
@@ -303,7 +189,7 @@ static int rpm_getcount(int tag)
 static char *rpm_getstr(int tag, int itemindex)
 {
 	rpm_index **found;
-	found = bsearch(&tag, mytags, tagcount, sizeof(struct rpmtag *), bsearch_rpmtag);
+	found = bsearch(&tag, G.mytags, G.tagcount, sizeof(struct rpmtag *), bsearch_rpmtag);
 	if (!found || itemindex >= found[0]->count)
 		return NULL;
 	if (found[0]->type == RPM_STRING_TYPE
@@ -311,7 +197,7 @@ static char *rpm_getstr(int tag, int itemindex)
 	 || found[0]->type == RPM_STRING_ARRAY_TYPE
 	) {
 		int n;
-		char *tmpstr = (char *) map + found[0]->offset;
+		char *tmpstr = (char *) G.map + found[0]->offset;
 		for (n = 0; n < itemindex; n++)
 			tmpstr = tmpstr + strlen(tmpstr) + 1;
 		return tmpstr;
@@ -322,32 +208,25 @@ static char *rpm_getstr(int tag, int itemindex)
 static int rpm_getint(int tag, int itemindex)
 {
 	rpm_index **found;
-	int *tmpint; /* NB: using int8_t* would be easier to code */
+	char *tmpint;
 
 	/* gcc throws warnings here when sizeof(void*)!=sizeof(int) ...
 	 * it's ok to ignore it because tag won't be used as a pointer */
-	found = bsearch(&tag, mytags, tagcount, sizeof(struct rpmtag *), bsearch_rpmtag);
+	found = bsearch(&tag, G.mytags, G.tagcount, sizeof(struct rpmtag *), bsearch_rpmtag);
 	if (!found || itemindex >= found[0]->count)
 		return -1;
 
-	tmpint = (int *) ((char *) map + found[0]->offset);
-
+	tmpint = (char *) G.map + found[0]->offset;
 	if (found[0]->type == RPM_INT32_TYPE) {
-		tmpint = (int *) ((char *) tmpint + itemindex*4);
-		/*return ntohl(*tmpint);*/
-		/* int can be != int32_t */
+		tmpint += itemindex*4;
 		return ntohl(*(int32_t*)tmpint);
 	}
 	if (found[0]->type == RPM_INT16_TYPE) {
-		tmpint = (int *) ((char *) tmpint + itemindex*2);
-		/* ??? read int, and THEN ntohs() it?? */
-		/*return ntohs(*tmpint);*/
+		tmpint += itemindex*2;
 		return ntohs(*(int16_t*)tmpint);
 	}
 	if (found[0]->type == RPM_INT8_TYPE) {
-		tmpint = (int *) ((char *) tmpint + itemindex);
-		/* ??? why we don't read byte here??? */
-		/*return ntohs(*tmpint);*/
+		tmpint += itemindex;
 		return *(int8_t*)tmpint;
 	}
 	return -1;
@@ -392,3 +271,134 @@ static void loop_through_files(int filetag, void (*fileaction)(char *filename, i
 		free(filename);
 	}
 }
+
+int rpm_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
+int rpm_main(int argc, char **argv)
+{
+	int opt, func = 0;
+	const unsigned pagesize = getpagesize();
+
+	while ((opt = getopt(argc, argv, "iqpldc")) != -1) {
+		switch (opt) {
+		case 'i': /* First arg: Install mode, with q: Information */
+			if (!func) func = rpm_install;
+			else func |= rpm_query_info;
+			break;
+		case 'q': /* First arg: Query mode */
+			if (func) bb_show_usage();
+			func = rpm_query;
+			break;
+		case 'p': /* Query a package */
+			func |= rpm_query_package;
+			break;
+		case 'l': /* List files in a package */
+			func |= rpm_query_list;
+			break;
+		case 'd': /* List doc files in a package (implies list) */
+			func |= rpm_query_list;
+			func |= rpm_query_list_doc;
+			break;
+		case 'c': /* List config files in a package (implies list) */
+			func |= rpm_query_list;
+			func |= rpm_query_list_config;
+			break;
+		default:
+			bb_show_usage();
+		}
+	}
+	argv += optind;
+	//argc -= optind;
+	if (!argv[0]) {
+		bb_show_usage();
+	}
+
+	while (*argv) {
+		int rpm_fd;
+		unsigned mapsize;
+		const char *source_rpm;
+
+		rpm_fd = xopen(*argv++, O_RDONLY);
+		G.mytags = rpm_gettags(rpm_fd, &G.tagcount);
+		if (!G.mytags)
+			bb_error_msg_and_die("error reading rpm header");
+		mapsize = xlseek(rpm_fd, 0, SEEK_CUR);
+		mapsize = (mapsize + pagesize) & -(int)pagesize;
+		/* Some NOMMU systems prefer MAP_PRIVATE over MAP_SHARED */
+		G.map = mmap(0, mapsize, PROT_READ, MAP_PRIVATE, rpm_fd, 0);
+//FIXME: error check?
+
+		source_rpm = rpm_getstr(TAG_SOURCERPM, 0);
+
+		if (func & rpm_install) {
+			/* Backup any config files */
+			loop_through_files(TAG_BASENAMES, fileaction_dobackup);
+			/* Extact the archive */
+			extract_cpio(rpm_fd, source_rpm);
+			/* Set the correct file uid/gid's */
+			loop_through_files(TAG_BASENAMES, fileaction_setowngrp);
+		}
+		else if ((func & (rpm_query|rpm_query_package)) == (rpm_query|rpm_query_package)) {
+			if (!(func & (rpm_query_info|rpm_query_list))) {
+				/* If just a straight query, just give package name */
+				printf("%s-%s-%s\n", rpm_getstr(TAG_NAME, 0), rpm_getstr(TAG_VERSION, 0), rpm_getstr(TAG_RELEASE, 0));
+			}
+			if (func & rpm_query_info) {
+				/* Do the nice printout */
+				time_t bdate_time;
+				struct tm *bdate_ptm;
+				char bdatestring[50];
+				const char *p;
+
+				printf("%-12s: %s\n", "Name"        , rpm_getstr(TAG_NAME, 0));
+				/* TODO compat: add "Epoch" here */
+				printf("%-12s: %s\n", "Version"     , rpm_getstr(TAG_VERSION, 0));
+				printf("%-12s: %s\n", "Release"     , rpm_getstr(TAG_RELEASE, 0));
+				/* add "Architecture" */
+				printf("%-12s: %s\n", "Install Date", "(not installed)");
+				printf("%-12s: %s\n", "Group"       , rpm_getstr(TAG_GROUP, 0));
+				printf("%-12s: %d\n", "Size"        , rpm_getint(TAG_SIZE, 0));
+				printf("%-12s: %s\n", "License"     , rpm_getstr(TAG_LICENSE, 0));
+				/* add "Signature" */
+				printf("%-12s: %s\n", "Source RPM"  , source_rpm ? source_rpm : "(none)");
+				bdate_time = rpm_getint(TAG_BUILDTIME, 0);
+				bdate_ptm = localtime(&bdate_time);
+				strftime(bdatestring, 50, "%a %d %b %Y %T %Z", bdate_ptm);
+				printf("%-12s: %s\n", "Build Date"  , bdatestring);
+				printf("%-12s: %s\n", "Build Host"  , rpm_getstr(TAG_BUILDHOST, 0));
+				p = rpm_getstr(TAG_PREFIXS, 0);
+				printf("%-12s: %s\n", "Relocations" , p ? p : "(not relocatable)");
+				/* add "Packager" */
+				p = rpm_getstr(TAG_VENDOR, 0);
+				printf("%-12s: %s\n", "Vendor"      , p ? p : "(none)");
+				printf("%-12s: %s\n", "URL"         , rpm_getstr(TAG_URL, 0));
+				printf("%-12s: %s\n", "Summary"     , rpm_getstr(TAG_SUMMARY, 0));
+				printf("Description :\n%s\n", rpm_getstr(TAG_DESCRIPTION, 0));
+			}
+			if (func & rpm_query_list) {
+				int count, it, flags;
+				count = rpm_getcount(TAG_BASENAMES);
+				for (it = 0; it < count; it++) {
+					flags = rpm_getint(TAG_FILEFLAGS, it);
+					switch (func & (rpm_query_list_doc|rpm_query_list_config)) {
+					case rpm_query_list_doc:
+						if (!(flags & RPMFILE_DOC)) continue;
+						break;
+					case rpm_query_list_config:
+						if (!(flags & RPMFILE_CONFIG)) continue;
+						break;
+					case rpm_query_list_doc|rpm_query_list_config:
+						if (!(flags & (RPMFILE_CONFIG|RPMFILE_DOC))) continue;
+						break;
+					}
+					printf("%s%s\n",
+						rpm_getstr(TAG_DIRNAMES, rpm_getint(TAG_DIRINDEXES, it)),
+						rpm_getstr(TAG_BASENAMES, it));
+				}
+			}
+		}
+		munmap(G.map, mapsize);
+		free(G.mytags);
+		close(rpm_fd);
+	}
+	return 0;
+}
