@@ -129,6 +129,9 @@
 //usage:     "\n	[YYYY.]MM.DD-hh:mm[:ss]"
 //usage:     "\n	YYYY-MM-DD hh:mm[:ss]"
 //usage:     "\n	[[[[[YY]YY]MM]DD]hh]mm[.ss]"
+//usage:	IF_FEATURE_DATE_COMPAT(
+//usage:     "\n	'date TIME' form accepts MMDDhhmm[[YY]YY][.ss] instead"
+//usage:	)
 //usage:
 //usage:#define date_example_usage
 //usage:       "$ date\n"
@@ -370,7 +373,7 @@ int date_main(int argc UNUSED_PARAM, char **argv)
 		date_buf[0] = '\0';
 	} else {
 		/* Handle special conversions */
-		if (strncmp(fmt_dt2str, "%f", 2) == 0) {
+		if (is_prefixed_with(fmt_dt2str, "%f")) {
 			fmt_dt2str = (char*)"%Y.%m.%d-%H:%M:%S";
 		}
 		/* Generate output string */
