@@ -49,16 +49,18 @@
  */
 
 //usage:#define uname_trivial_usage
-//usage:       "[-amnrspv]"
+//usage:       "[-amnrspvio]"
 //usage:#define uname_full_usage "\n\n"
 //usage:       "Print system information\n"
 //usage:     "\n	-a	Print all"
 //usage:     "\n	-m	The machine (hardware) type"
 //usage:     "\n	-n	Hostname"
-//usage:     "\n	-r	OS release"
-//usage:     "\n	-s	OS name (default)"
+//usage:     "\n	-r	Kernel release"
+//usage:     "\n	-s	Kernel name (default)"
 //usage:     "\n	-p	Processor type"
-//usage:     "\n	-v	OS version"
+//usage:     "\n	-v	Kernel version"
+//usage:     "\n	-i	The hardware platform"
+//usage:     "\n	-o	OS name"
 //usage:
 //usage:#define uname_example_usage
 //usage:       "$ uname -a\n"
@@ -72,7 +74,7 @@ typedef struct {
 	struct utsname name;
 	char processor[sizeof(((struct utsname*)NULL)->machine)];
 	char platform[sizeof(((struct utsname*)NULL)->machine)];
-	char os[sizeof("GNU/Linux")];
+	char os[sizeof(CONFIG_UNAME_OSNAME)];
 } uname_info_t;
 
 static const char options[] ALIGN1 = "snrvmpioa";
@@ -139,7 +141,7 @@ int uname_main(int argc UNUSED_PARAM, char **argv)
 #endif
 	strcpy(uname_info.processor, unknown_str);
 	strcpy(uname_info.platform, unknown_str);
-	strcpy(uname_info.os, "GNU/Linux");
+	strcpy(uname_info.os, CONFIG_UNAME_OSNAME);
 #if 0
 	/* Fedora does something like this */
 	strcpy(uname_info.processor, uname_info.name.machine);
