@@ -7,6 +7,18 @@
  * The "ed" built-in command (much simplified)
  */
 
+//config:config ED
+//config:	bool "ed"
+//config:	default y
+//config:	help
+//config:	  The original 1970's Unix text editor, from the days of teletypes.
+//config:	  Small, simple, evil. Part of SUSv3. If you're not already using
+//config:	  this, you don't need it.
+
+//kbuild:lib-$(CONFIG_ED) += ed.o
+
+//applet:IF_ED(APPLET(ed, BB_DIR_BIN, BB_SUID_DROP))
+
 //usage:#define ed_trivial_usage ""
 //usage:#define ed_full_usage ""
 
@@ -194,7 +206,7 @@ static void doCommands(void)
 				if (fileName)
 					printf("\"%s\"\n", fileName);
 				else
-					printf("No file name\n");
+					puts("No file name");
 				break;
 			}
 			free(fileName);
@@ -720,7 +732,6 @@ static int readLines(const char *file, int num)
 		cc = safe_read(fd, bufPtr, bufSize - bufUsed);
 		bufUsed += cc;
 		bufPtr = bufBase;
-
 	} while (cc > 0);
 
 	if (cc < 0) {
