@@ -373,6 +373,11 @@ static int grep_file(FILE *file)
  opt_f_not_found: ;
 				}
 			} else {
+#if ENABLE_EXTRA_COMPAT
+				unsigned start_pos;
+#else
+				int match_flg;
+#endif
 				char *match_at;
 
 				if (!(gl->flg_mem_alocated_compiled & COMPILED)) {
@@ -389,15 +394,19 @@ static int grep_file(FILE *file)
 #if !ENABLE_EXTRA_COMPAT
 				gl->matched_range.rm_so = 0;
 				gl->matched_range.rm_eo = 0;
+				match_flg = 0;
+#else
+				start_pos = 0;
 #endif
 				match_at = line;
  opt_w_again:
+//bb_error_msg("'%s' start_pos:%d line_len:%d", match_at, start_pos, line_len);
 				if (
 #if !ENABLE_EXTRA_COMPAT
-					regexec(&gl->compiled_regex, match_at, 1, &gl->matched_range, 0) == 0
+					regexec(&gl->compiled_regex, match_at, 1, &gl->matched_range, match_flg) == 0
 #else
 					re_search(&gl->compiled_regex, match_at, line_len,
-							/*start:*/ 0, /*range:*/ line_len,
+							start_pos, /*range:*/ line_len,
 							&gl->matched_range) >= 0
 #endif
 				) {
@@ -409,16 +418,34 @@ static int grep_file(FILE *file)
 						found = 1;
 					} else {
 						char c = ' ';
-						if (gl->matched_range.rm_so)
+						if (match_at > line || gl->matched_range.rm_so != 0) {
 							c = match_at[gl->matched_range.rm_so - 1];
+						}
 						if (!isalnum(c) && c != '_') {
 							c = match_at[gl->matched_range.rm_eo];
-							if (!c || (!isalnum(c) && c != '_')) {
-								found = 1;
-							} else {
+						}
+						if (!isalnum(c) && c != '_') {
+							found = 1;
+						} else {
+			/*
+			 * Why check gl->matched_range.rm_eo?
+			 * Zero-length match makes -w skip the line:
+			 * "echo foo | grep ^" prints "foo",
+			 * "echo foo | grep -w ^" prints nothing.
+			 * Without such check, we can loop forever.
+			 */
+#if !ENABLE_EXTRA_COMPAT
+							if (gl->matched_range.rm_eo != 0) {
 								match_at += gl->matched_range.rm_eo;
+								match_flg |= REG_NOTBOL;
 								goto opt_w_again;
 							}
+#else
+							if (gl->matched_range.rm_eo > start_pos) {
+								start_pos = gl->matched_range.rm_eo;
+								goto opt_w_again;
+							}
+#endif
 						}
 					}
 				}
@@ -641,7 +668,7 @@ static int grep_dir(const char *dir)
 	int matched = 0;
 	recursive_action(dir,
 		/* recurse=yes */ ACTION_RECURSE |
-		/* followLinks=no */
+		/* followLinks=command line only */ ACTION_FOLLOWLINKS_L0 |
 		/* depthFirst=yes */ ACTION_DEPTHFIRST,
 		/* fileAction= */ file_action_grep,
 		/* dirAction= */ NULL,
