@@ -2,30 +2,24 @@
 /*
  * Mini sulogin implementation for busybox
  *
- * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
+ * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 
+//usage:#define sulogin_trivial_usage
+//usage:       "[-t N] [TTY]"
+//usage:#define sulogin_full_usage "\n\n"
+//usage:       "Single user login\n"
+//usage:     "\n	-t N	Timeout"
+
 #include "libbb.h"
 #include <syslog.h>
 
-//static void catchalarm(int UNUSED_PARAM junk)
-//{
-//	exit(EXIT_FAILURE);
-//}
-
-
 int sulogin_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int sulogin_main(int argc UNUSED_PARAM, char **argv)
 {
-	char *cp;
 	int timeout = 0;
 	struct passwd *pwd;
 	const char *shell;
-#if ENABLE_FEATURE_SHADOWPASSWDS
-	/* Using _r function to avoid pulling in static buffers */
-	char buffer[256];
-	struct spwd spw;
-#endif
 
 	logmode = LOGMODE_BOTH;
 	openlog(applet_name, 0, LOG_AUTH);
@@ -56,43 +50,24 @@ int sulogin_main(int argc UNUSED_PARAM, char **argv)
 		goto auth_error;
 	}
 
-#if ENABLE_FEATURE_SHADOWPASSWDS
-	{
-		/* getspnam_r may return 0 yet set result to NULL.
-		 * At least glibc 2.4 does this. Be extra paranoid here. */
-		struct spwd *result = NULL;
-		int r = getspnam_r(pwd->pw_name, &spw, buffer, sizeof(buffer), &result);
-		if (r || !result) {
-			goto auth_error;
-		}
-		pwd->pw_passwd = result->sp_pwdp;
-	}
-#endif
-
 	while (1) {
-		char *encrypted;
 		int r;
 
-		/* cp points to a static buffer that is zeroed every time */
-		cp = bb_ask(STDIN_FILENO, timeout,
-				"Give root password for system maintenance\n"
-				"(or type Control-D for normal startup):");
-
-		if (!cp || !*cp) {
+		r = ask_and_check_password_extended(pwd, timeout,
+			"Give root password for system maintenance\n"
+			"(or type Control-D for normal startup):"
+		);
+		if (r < 0) {
+			/* ^D, ^C, timeout, or read error */
 			bb_info_msg("Normal startup");
 			return 0;
 		}
-		encrypted = pw_encrypt(cp, pwd->pw_passwd, 1);
-		r = strcmp(encrypted, pwd->pw_passwd);
-		free(encrypted);
-		if (r == 0) {
+		if (r > 0) {
 			break;
 		}
-		bb_do_delay(FAIL_DELAY);
-		bb_error_msg("login incorrect");
+		bb_do_delay(LOGIN_FAIL_DELAY);
+		bb_info_msg("Login incorrect");
 	}
-	memset(cp, 0, strlen(cp));
-//	signal(SIGALRM, SIG_DFL);
 
 	bb_info_msg("System Maintenance Mode");
 
