@@ -6,6 +6,22 @@
  * Busybox port: Nick Fedchik <nick@fedchik.org.ua>
  */
 
+//usage:#define arping_trivial_usage
+//usage:       "[-fqbDUA] [-c CNT] [-w TIMEOUT] [-I IFACE] [-s SRC_IP] DST_IP"
+//usage:#define arping_full_usage "\n\n"
+//usage:       "Send ARP requests/replies\n"
+//usage:     "\n	-f		Quit on first ARP reply"
+//usage:     "\n	-q		Quiet"
+//usage:     "\n	-b		Keep broadcasting, don't go unicast"
+//usage:     "\n	-D		Exit with 1 if DST_IP replies"
+//usage:     "\n	-U		Unsolicited ARP mode, update your neighbors"
+//usage:     "\n	-A		ARP answer mode, update your neighbors"
+//usage:     "\n	-c N		Stop after sending N ARP requests"
+//usage:     "\n	-w TIMEOUT	Seconds to wait for ARP reply"
+//usage:     "\n	-I IFACE	Interface to use (default eth0)"
+//usage:     "\n	-s SRC_IP	Sender IP address"
+//usage:     "\n	DST_IP		Target IP address"
+
 #include <arpa/inet.h>
 #include <net/if.h>
 #include <netinet/ether.h>
@@ -146,7 +162,7 @@ static void catcher(void)
 	alarm(1);
 }
 
-static bool recv_pack(unsigned char *buf, int len, struct sockaddr_ll *FROM)
+static void recv_pack(unsigned char *buf, int len, struct sockaddr_ll *FROM)
 {
 	struct arphdr *ah = (struct arphdr *) buf;
 	unsigned char *p = (unsigned char *) (ah + 1);
@@ -165,33 +181,33 @@ static bool recv_pack(unsigned char *buf, int len, struct sockaddr_ll *FROM)
 	if (FROM->sll_pkttype != PACKET_HOST
 	 && FROM->sll_pkttype != PACKET_BROADCAST
 	 && FROM->sll_pkttype != PACKET_MULTICAST)
-		return false;
+		return;
 
 	/* Only these types are recognized */
 	if (ah->ar_op != htons(ARPOP_REQUEST) && ah->ar_op != htons(ARPOP_REPLY))
-		return false;
+		return;
 
 	/* ARPHRD check and this darned FDDI hack here :-( */
 	if (ah->ar_hrd != htons(FROM->sll_hatype)
 	 && (FROM->sll_hatype != ARPHRD_FDDI || ah->ar_hrd != htons(ARPHRD_ETHER)))
-		return false;
+		return;
 
 	/* Protocol must be IP. */
 	if (ah->ar_pro != htons(ETH_P_IP)
 	 || (ah->ar_pln != 4)
 	 || (ah->ar_hln != me.sll_halen)
 	 || (len < (int)(sizeof(*ah) + 2 * (4 + ah->ar_hln))))
-		return false;
+		return;
 
 	move_from_unaligned32(src_ip.s_addr, p + ah->ar_hln);
 	move_from_unaligned32(dst_ip.s_addr, p + ah->ar_hln + 4 + ah->ar_hln);
 
 	if (dst.s_addr != src_ip.s_addr)
-		return false;
+		return;
 	if (!(option_mask32 & DAD)) {
 		if ((src.s_addr != dst_ip.s_addr)
-			|| (memcmp(p + ah->ar_hln + 4, &me.sll_addr, ah->ar_hln)))
-			return false;
+		 || (memcmp(p + ah->ar_hln + 4, &me.sll_addr, ah->ar_hln)))
+			return;
 	} else {
 		/* DAD packet was:
 		   src_ip = 0 (or some src)
@@ -208,7 +224,7 @@ static bool recv_pack(unsigned char *buf, int len, struct sockaddr_ll *FROM)
 		 */
 		if ((memcmp(p, &me.sll_addr, me.sll_halen) == 0)
 		 || (src.s_addr && src.s_addr != dst_ip.s_addr))
-			return false;
+			return;
 	}
 	if (!(option_mask32 & QUIET)) {
 		int s_printed = 0;
@@ -233,7 +249,7 @@ static bool recv_pack(unsigned char *buf, int len, struct sockaddr_ll *FROM)
 			unsigned diff = MONOTONIC_US() - last;
 			printf(" %u.%03ums\n", diff / 1000, diff % 1000);
 		} else {
-			printf(" UNSOLICITED?\n");
+			puts(" UNSOLICITED?");
 		}
 		fflush_all();
 	}
@@ -248,7 +264,6 @@ static bool recv_pack(unsigned char *buf, int len, struct sockaddr_ll *FROM)
 		memcpy(he.sll_addr, p, me.sll_halen);
 		option_mask32 |= UNICASTING;
 	}
-	return true;
 }
 
 int arping_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
@@ -268,7 +283,6 @@ int arping_main(int argc UNUSED_PARAM, char **argv)
 	// Need to remove SUID_NEVER from applets.h for this to work
 	//xsetuid(getuid());
 
-	err_str = xasprintf("interface %s %%s", device);
 	{
 		unsigned opt;
 		char *str_timeout;
@@ -286,7 +300,7 @@ int arping_main(int argc UNUSED_PARAM, char **argv)
 	}
 
 	target = argv[optind];
-
+	err_str = xasprintf("interface %s %%s", device);
 	xfunc_error_retval = 2;
 
 	{
@@ -343,8 +357,8 @@ int arping_main(int argc UNUSED_PARAM, char **argv)
 			saddr.sin_port = htons(1025);
 			saddr.sin_addr = dst;
 
-			if (setsockopt(probe_fd, SOL_SOCKET, SO_DONTROUTE, &const_int_1, sizeof(const_int_1)) == -1)
-				bb_perror_msg("setsockopt(SO_DONTROUTE)");
+			if (setsockopt_SOL_SOCKET_1(probe_fd, SO_DONTROUTE) != 0)
+				bb_perror_msg("setsockopt(%s)", "SO_DONTROUTE");
 			xconnect(probe_fd, (struct sockaddr *) &saddr, sizeof(saddr));
 			getsockname(probe_fd, (struct sockaddr *) &saddr, &alen);
 			//never happens:
