@@ -137,12 +137,11 @@ static NOINLINE int print_linkinfo(const struct nlmsghdr *n)
 	{
 		unsigned m_flag = 0;
 		if (tb[IFLA_LINK]) {
-			SPRINT_BUF(b1);
 			int iflink = *(int*)RTA_DATA(tb[IFLA_LINK]);
 			if (iflink == 0)
 				printf("@NONE: ");
 			else {
-				printf("@%s: ", ll_idx_n2a(iflink, b1));
+				printf("@%s: ", ll_index_to_name(iflink));
 				m_flag = ll_index_to_flags(iflink);
 				m_flag = !(m_flag & IFF_UP);
 			}
@@ -158,8 +157,7 @@ static NOINLINE int print_linkinfo(const struct nlmsghdr *n)
 		printf("qdisc %s ", (char*)RTA_DATA(tb[IFLA_QDISC]));
 #ifdef IFLA_MASTER
 	if (tb[IFLA_MASTER]) {
-		SPRINT_BUF(b1);
-		printf("master %s ", ll_idx_n2a(*(int*)RTA_DATA(tb[IFLA_MASTER]), b1));
+		printf("master %s ", ll_index_to_name(*(int*)RTA_DATA(tb[IFLA_MASTER])));
 	}
 #endif
 /* IFLA_OPERSTATE was added to kernel with the same commit as IFF_DORMANT */
@@ -218,7 +216,6 @@ static int FAST_FUNC print_addrinfo(const struct sockaddr_nl *who UNUSED_PARAM,
 	int len = n->nlmsg_len;
 	struct rtattr * rta_tb[IFA_MAX+1];
 	char abuf[256];
-	SPRINT_BUF(b1);
 
 	if (n->nlmsg_type != RTM_NEWADDR && n->nlmsg_type != RTM_DELADDR)
 		return 0;
@@ -250,7 +247,7 @@ static int FAST_FUNC print_addrinfo(const struct sockaddr_nl *who UNUSED_PARAM,
 		if (rta_tb[IFA_LABEL])
 			label = RTA_DATA(rta_tb[IFA_LABEL]);
 		else
-			label = ll_idx_n2a(ifa->ifa_index, b1);
+			label = ll_index_to_name(ifa->ifa_index);
 		if (fnmatch(G_filter.label, label, 0) != 0)
 			return 0;
 	}
@@ -325,7 +322,7 @@ static int FAST_FUNC print_addrinfo(const struct sockaddr_nl *who UNUSED_PARAM,
 					abuf, sizeof(abuf))
 		);
 	}
-	printf("scope %s ", rtnl_rtscope_n2a(ifa->ifa_scope, b1));
+	printf("scope %s ", rtnl_rtscope_n2a(ifa->ifa_scope));
 	if (ifa->ifa_flags & IFA_F_SECONDARY) {
 		ifa->ifa_flags &= ~IFA_F_SECONDARY;
 		printf("secondary ");
@@ -556,12 +553,11 @@ int FAST_FUNC ipaddr_list_or_flush(char **argv, int flush)
 							continue;
 					}
 					if (G_filter.label) {
-						SPRINT_BUF(b1);
 						const char *label;
 						if (tb[IFA_LABEL])
 							label = RTA_DATA(tb[IFA_LABEL]);
 						else
-							label = ll_idx_n2a(ifa->ifa_index, b1);
+							label = ll_index_to_name(ifa->ifa_index);
 						if (fnmatch(G_filter.label, label, 0) != 0)
 							continue;
 					}
@@ -598,7 +594,7 @@ static int default_scope(inet_prefix *lcl)
 }
 
 /* Return value becomes exitcode. It's okay to not return at all */
-static int ipaddr_modify(int cmd, char **argv)
+static int ipaddr_modify(int cmd, int flags, char **argv)
 {
 	static const char option[] ALIGN1 =
 		"peer\0""remote\0""broadcast\0""brd\0"
@@ -622,15 +618,17 @@ static int ipaddr_modify(int cmd, char **argv)
 	memset(&req, 0, sizeof(req));
 
 	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifaddrmsg));
-	req.n.nlmsg_flags = NLM_F_REQUEST;
+	req.n.nlmsg_flags = NLM_F_REQUEST | flags;
 	req.n.nlmsg_type = cmd;
 	req.ifa.ifa_family = preferred_family;
 
 	while (*argv) {
-		const smalluint arg = index_in_strings(option, *argv);
-		if (arg <= 1) { /* peer, remote */
+		unsigned arg = index_in_strings(option, *argv);
+		/* if search fails, "local" is assumed */
+		if ((int)arg >= 0)
 			NEXT_ARG();
 
+		if (arg <= 1) { /* peer, remote */
 			if (peer_len) {
 				duparg("peer", *argv);
 			}
@@ -643,7 +641,6 @@ static int ipaddr_modify(int cmd, char **argv)
 			req.ifa.ifa_prefixlen = peer.bitlen;
 		} else if (arg <= 3) { /* broadcast, brd */
 			inet_prefix addr;
-			NEXT_ARG();
 			if (brd_len) {
 				duparg("broadcast", *argv);
 			}
@@ -660,7 +657,6 @@ static int ipaddr_modify(int cmd, char **argv)
 			}
 		} else if (arg == 4) { /* anycast */
 			inet_prefix addr;
-			NEXT_ARG();
 			if (any_len) {
 				duparg("anycast", *argv);
 			}
@@ -672,22 +668,18 @@ static int ipaddr_modify(int cmd, char **argv)
 			any_len = addr.bytelen;
 		} else if (arg == 5) { /* scope */
 			uint32_t scope = 0;
-			NEXT_ARG();
 			if (rtnl_rtscope_a2n(&scope, *argv)) {
 				invarg(*argv, "scope");
 			}
 			req.ifa.ifa_scope = scope;
 			scoped = 1;
 		} else if (arg == 6) { /* dev */
-			NEXT_ARG();
 			d = *argv;
 		} else if (arg == 7) { /* label */
-			NEXT_ARG();
 			l = *argv;
 			addattr_l(&req.n, sizeof(req), IFA_LABEL, l, strlen(l) + 1);
 		} else {
-			if (arg == 8) /* local */
-				NEXT_ARG();
+			/* local (specified or assumed) */
 			if (local_len) {
 				duparg2("local", *argv);
 			}
@@ -705,7 +697,7 @@ static int ipaddr_modify(int cmd, char **argv)
 		/* There was no "dev IFACE", but we need that */
 		bb_error_msg_and_die("need \"dev IFACE\"");
 	}
-	if (l && strncmp(d, l, strlen(d)) != 0) {
+	if (l && !is_prefixed_with(l, d)) {
 		bb_error_msg_and_die("\"dev\" (%s) must match \"label\" (%s)", d, l);
 	}
 
@@ -724,7 +716,7 @@ static int ipaddr_modify(int cmd, char **argv)
 		}
 		brd = peer;
 		if (brd.bitlen <= 30) {
-			for (i=31; i>=brd.bitlen; i--) {
+			for (i = 31; i >= brd.bitlen; i--) {
 				if (brd_len == -1)
 					brd.data[0] |= htonl(1<<(31-i));
 				else
@@ -753,16 +745,24 @@ static int ipaddr_modify(int cmd, char **argv)
 int FAST_FUNC do_ipaddr(char **argv)
 {
 	static const char commands[] ALIGN1 =
-		"add\0""delete\0""list\0""show\0""lst\0""flush\0";
-	smalluint cmd = 2;
+		/* 0    1         2      3          4         5       6       7      8 */
+		"add\0""change\0""chg\0""replace\0""delete\0""list\0""show\0""lst\0""flush\0";
+	int cmd = 2;
 	if (*argv) {
 		cmd = index_in_substrings(commands, *argv);
-		if (cmd > 5)
-			bb_error_msg_and_die(bb_msg_invalid_arg, *argv, applet_name);
+		if (cmd < 0)
+			invarg(*argv, applet_name);
 		argv++;
-		if (cmd <= 1)
-			return ipaddr_modify((cmd == 0) ? RTM_NEWADDR : RTM_DELADDR, argv);
+		if (cmd <= 4) {
+			return ipaddr_modify(
+				/*cmd:*/ cmd == 4 ? RTM_DELADDR : RTM_NEWADDR,
+				/*flags:*/
+					cmd == 0 ? NLM_F_CREATE|NLM_F_EXCL : /* add */
+					cmd == 1 || cmd == 2 ? NLM_F_REPLACE : /* change */
+					cmd == 3 ? NLM_F_CREATE|NLM_F_REPLACE : /* replace */
+					0 /* delete */
+			, argv);
+		}
 	}
-	/* 2 == list, 3 == show, 4 == lst */
-	return ipaddr_list_or_flush(argv, cmd == 5);
+	return ipaddr_list_or_flush(argv, cmd == 8);
 }
